@@ -1,4 +1,5 @@
 """Prepare user data for weight generation."""
+import logging
 import time
 from abc import ABC
 from abc import abstractmethod
@@ -10,9 +11,8 @@ from typing import Union
 import geopandas as gpd
 import numpy as np
 import pandas as pd
-import rioxarray as rxr
 import xarray as xr
-from pyproj.crs import CRS
+from pyproj import CRS
 
 from gdptools.data.agg_gen_data import AggData
 from gdptools.data.odap_cat_data import CatGrids
@@ -23,12 +23,18 @@ from gdptools.helpers import build_subset_tiff
 from gdptools.utils import _check_for_intersection
 from gdptools.utils import _check_for_intersection_nc
 from gdptools.utils import _get_cells_poly
+from gdptools.utils import _get_crs
 from gdptools.utils import _get_data_via_catalog
+from gdptools.utils import _get_geodataframe
+from gdptools.utils import _get_rxr_dataset
 from gdptools.utils import _get_shp_bounds_w_buffer
 from gdptools.utils import _get_shp_file
 from gdptools.utils import _get_top_to_bottom
+from gdptools.utils import _get_xr_dataset
 from gdptools.utils import _read_shp_file
 
+logger = logging.getLogger(__name__)
+
 
 class UserData(ABC):
     """Prepare data for different sources for weight generation."""
@@ -44,7 +50,7 @@ class UserData(ABC):
         pass
 
     @abstractmethod
-    def prep_agg_data(self) -> AggData:
+    def prep_agg_data(self, key: str) -> AggData:
         """Abstract method for preparing data for aggregation."""
         pass
 
@@ -65,8 +71,8 @@ class ODAPCatData(UserData):
     def __init__(
         self,
         *,
-        param_dict: dict[str, dict],
-        grid_dict: dict[str, dict],
+        param_dict: dict[str, dict[str, Any]],
+        grid_dict: dict[str, dict[str, Any]],
         f_feature: Union[str, Path, gpd.GeoDataFrame],
         id_feature: str,
         period: List[str],
@@ -82,14 +88,30 @@ class ODAPCatData(UserData):
             period (List[str]): List containing date strings defining start and end
                 time slice for aggregation.
         """
+        logger.info("Initializing ODAPCatData")
+        logger.info("  - loading data")
         self.param_dict = param_dict
         self.grid_dict = grid_dict
         self.f_feature = f_feature
         self.id_feature = id_feature
         self.period = pd.to_datetime(period)
-        self._heck_input_dicts()
+        self._check_input_dicts()
         self.gdf = _read_shp_file(self.f_feature)
 
+        self._check_id_feature()
+        self.keys = self.get_vars()
+        self._check_input_dicts
+        cp = self._create_catparam(key=self.keys[0])
+        cg = self._create_catgrid(key=self.keys[0])
+        logger.info("  - checking latitude bounds")
+        is_intersect, is_degrees, is_0_360 = _check_for_intersection(
+            cat_params=cp, cat_grid=cg, gdf=self.gdf
+        )
+        self.gdf, self.gdf_bounds = _get_shp_file(
+            shp_file=self.gdf, cat_grid=cg, is_degrees=is_degrees
+        )
+        self.rotate_ds = bool((not is_intersect) & is_degrees & (not is_0_360))
+
     def get_feature_id(self) -> str:
         """Return id_feature."""
         return self.id_feature
@@ -110,29 +132,20 @@ class ODAPCatData(UserData):
         cp = self._create_catparam(key=key)
         cg = self._create_catgrid(key=key)
 
-        is_intersect, is_degrees, is_0_360 = _check_for_intersection(
-            cat_params=cp, cat_grid=cg, gdf=self.gdf
-        )
-
-        gdf, gdf_bounds = _get_shp_file(
-            shp_file=self.gdf, cat_grid=cg, is_degrees=is_degrees
-        )
-
-        rotate_ds = bool((not is_intersect) & is_degrees & (not is_0_360))
         ds_ss = _get_data_via_catalog(
             cat_params=cp,
             cat_grid=cg,
-            bounds=gdf_bounds,
-            begin_date=self.period[0],
-            end_date=self.period[1],
-            rotate_lon=rotate_ds,
+            bounds=self.gdf_bounds,
+            begin_date=str(self.period[0]),
+            end_date=str(self.period[1]),
+            rotate_lon=self.rotate_ds,
         )
         return AggData(
             variable=key,
             cat_param=cp,
             cat_grid=cg,
             da=ds_ss,
-            feature=gdf,
+            feature=self.gdf,
             id_feature=self.id_feature,
             period=self.period,
         )
@@ -141,37 +154,18 @@ class ODAPCatData(UserData):
         """Prepare and return WeightData for weight generation."""
         # For weight generation we just need param and grid dict from one variable
         # passed to CatParms and CatGrids.
-        self._heck_input_dicts()
-        keys = self.get_vars()
-        cp = self._create_catparam(key=keys[0])
-        cg = self._create_catgrid(key=keys[0])
-        gdf = _read_shp_file(self.f_feature)
-        self._check_id_feature(gdf)
-        is_intersect, is_degrees, is_0_360 = _check_for_intersection(
-            cat_params=cp, cat_grid=cg, gdf=gdf
-        )
-
-        gdf, gdf_bounds = _get_shp_file(
-            shp_file=gdf, cat_grid=cg, is_degrees=is_degrees
-        )
-
-        rotate_ds = bool((not is_intersect) & is_degrees & (not is_0_360))
-        ds_ss = _get_data_via_catalog(
+        cp = self._create_catparam(key=self.keys[0])
+        cg = self._create_catgrid(key=self.keys[0])
+        self.ds_ss = _get_data_via_catalog(
             cat_params=cp,
             cat_grid=cg,
-            bounds=gdf_bounds,
-            begin_date=self.period[0],
-            rotate_lon=rotate_ds,
+            bounds=self.gdf_bounds,
+            begin_date=str(self.period[0]),
+            rotate_lon=self.rotate_ds,
         )
-
-        # tsrt = time.perf_counter()
-        # ds_ss.load()
-        # tend = time.perf_counter()
-        # print(f"loading {cp.varname} in {tend-tsrt:0.4f} seconds")
-
         tsrt = time.perf_counter()
         gdf_grid = _get_cells_poly(
-            xr_a=ds_ss,
+            xr_a=self.ds_ss,
             x=cg.X_name,
             y=cg.Y_name,
             crs_in=cg.proj,
@@ -179,24 +173,26 @@ class ODAPCatData(UserData):
         tend = time.perf_counter()
         print(f"grid cells generated in {tend-tsrt:0.4f} seconds")
 
-        return WeightData(feature=gdf, id_feature=self.id_feature, grid_cells=gdf_grid)
+        return WeightData(
+            feature=self.gdf, id_feature=self.id_feature, grid_cells=gdf_grid
+        )
 
-    def _check_id_feature(self, gdf):
+    def _check_id_feature(self: "ODAPCatData") -> None:
         """Check id_feature in gdf."""
-        if self.id_feature not in gdf.columns[:]:
+        if self.id_feature not in self.gdf.columns[:]:
             raise ValueError(
-                f"shp_poly_idx: {self.id_feature} not in gdf columns: {gdf.columns}"
+                f"shp_poly_idx: {self.id_feature} not in gdf columns: {self.gdf.columns}"
             )
 
-    def _create_catgrid(self, key: str):
+    def _create_catgrid(self: "ODAPCatData", key: str) -> CatGrids:
         """Create CatGrids."""
-        return CatGrids(**self.grid_dict.get(key))
+        return CatGrids(**self.grid_dict.get(key))  # type: ignore
 
-    def _create_catparam(self, key: str):
+    def _create_catparam(self: "ODAPCatData", key: str) -> CatParams:
         """Create CatParams."""
-        return CatParams(**self.param_dict.get(key))
+        return CatParams(**self.param_dict.get(key))  # type: ignore
 
-    def _heck_input_dicts(self):
+    def _check_input_dicts(self: "ODAPCatData") -> None:
         """Check input dicts are of equal length and with at least one entry."""
         if len(self.param_dict) != len(self.grid_dict):
             raise ValueError(
@@ -212,18 +208,20 @@ class UserCatData(UserData):
 
     def __init__(
         self,
-        ds: Union[str, Path, xr.Dataset],
+        ds: Union[str, xr.Dataset],
         proj_ds: Any,
         x_coord: str,
         y_coord: str,
         t_coord: str,
         var: List[str],
         f_feature: Union[str, Path, gpd.GeoDataFrame],
-        proj_feature: str,
+        proj_feature: Any,
         id_feature: str,
         period: List[str],
     ) -> None:
-        """Contains data preperation methods based on UserData.
+        """__init__ Contains data preperation methods based on UserData.
+
+        _extended_summary_
 
         Args:
             ds (Union[str, Path, xr.Dataset]): _description_
@@ -233,15 +231,18 @@ class UserCatData(UserData):
             t_coord (str): _description_
             var (List[str]): _description_
             f_feature (Union[str, Path, gpd.GeoDataFrame]): _description_
-            proj_feature (str): _description_
+            proj_feature (Any): _description_
             id_feature (str): _description_
-            period (List[str]): List of 2 datetime strings representing start and end of
-                period of interest.
+            period (List[str]): _description_
 
         Raises:
             TypeError: _description_
+            ValueError: _description_
         """
-        self.ds = ds
+        logger.info("Initializing UserCatData")
+        logger.info("  - loading data")
+        self.ds = _get_xr_dataset(ds=ds)
+        self.f_feature = _get_geodataframe(f_feature=f_feature)
         self.proj_ds = proj_ds
         self.x_coord = x_coord
         self.y_coord = y_coord
@@ -251,10 +252,57 @@ class UserCatData(UserData):
         if not isinstance(var, List):
             raise TypeError(f"Input var {var} should be a list of variables")
         self.var = var
-        self.f_feature = f_feature
+        tcrs = _get_crs(self.f_feature.crs)
         self.proj_feature = proj_feature
+        spcrs = _get_crs(self.proj_feature)
+        if tcrs != spcrs:
+            raise ValueError(f"f_feature.crs {tcrs} not equal to proj.feature {spcrs}")
         self.id_feature = id_feature
 
+        self.gdf_bounds = _get_shp_bounds_w_buffer(
+            gdf=self.f_feature,
+            ds=self.ds,
+            crs=self.proj_ds,
+            lon=self.x_coord,
+            lat=self.y_coord,
+        )
+        logger.info("  - checking latitude bounds")
+        is_intersect, is_degrees, is_0_360 = _check_for_intersection_nc(
+            ds=self.ds,
+            x_name=self.x_coord,
+            y_name=self.y_coord,
+            proj=self.proj_ds,
+            gdf=self.f_feature,
+        )
+
+        if bool((not is_intersect) & is_degrees & (not is_0_360)):  # rotate
+            logger.info("  - rotating into -180 - 180")
+            self.ds.coords[self.x_coord] = (
+                self.ds.coords[self.x_coord] + 180
+            ) % 360 - 180
+            self.ds = self.ds.sortby(self.ds[self.x_coord])
+
+        # calculate toptobottom (order of latitude coords)
+        self.ttb = _get_top_to_bottom(self.ds, self.y_coord)
+        logger.info("  - getting gridded data subset")
+        self.weight_subset_dict = build_subset(
+            bounds=self.gdf_bounds,
+            xname=self.x_coord,
+            yname=self.y_coord,
+            tname=self.t_coord,
+            toptobottom=self.ttb,
+            date_min=self.period[0],
+        )
+        self.agg_subset_dict = build_subset(
+            bounds=self.gdf_bounds,
+            xname=self.x_coord,
+            yname=self.y_coord,
+            tname=self.t_coord,
+            toptobottom=self.ttb,
+            date_min=self.period[0],
+            date_max=self.period[1],
+        )
+
     @classmethod
     def __repr__(cls) -> str:
         """Print class name."""
@@ -270,63 +318,28 @@ class UserCatData(UserData):
 
     def prep_agg_data(self, key: str) -> AggData:
         """Prep AggData from UserData."""
-        ds = self._get_xr_dataset()
-
-        gdf_in = self._get_geodataframe()
-
-        gdf_in.set_crs(self.proj_feature, inplace=True)
-
-        gdf_bounds = _get_shp_bounds_w_buffer(
-            gdf=gdf_in, ds=ds, crs=self.proj_ds, lon=self.x_coord, lat=self.y_coord
-        )
-
-        is_intersect, is_degrees, is_0_360 = _check_for_intersection_nc(
-            ds=ds,
-            x_name=self.x_coord,
-            y_name=self.y_coord,
-            proj=self.proj_ds,
-            gdf=gdf_in,
-        )
-
-        if bool((not is_intersect) & is_degrees & (not is_0_360)):  # rotate
-            ds.coords[self.x_coord] = (ds.coords[self.x_coord] + 180) % 360 - 180
-            ds = ds.sortby(ds[self.x_coord])
-
-        # calculate toptobottom (order of latitude coords)
-        ttb = _get_top_to_bottom(ds, self.y_coord)
-
-        subset_dict = build_subset(
-            bounds=gdf_bounds,
-            xname=self.x_coord,
-            yname=self.y_coord,
-            tname=self.t_coord,
-            toptobottom=ttb,
-            date_min=self.period[0],
-            date_max=self.period[1],
-        )
-
-        data_ss = ds[key].sel(**subset_dict)
+        logger.info("Agg Data preparation - beginning")
+        data_ss: xr.DataArray = self.ds[key].sel(**self.agg_subset_dict)  # type: ignore
         cplist = self._get_user_catparams(key=key, da=data_ss)
         cglist = self._get_user_catgrids(key=key, da=data_ss)
-
+        logger.info("  - creating param and grid dicts")
         param_dict = dict(zip([key], cplist))  # noqa B905
         grid_dict = dict(zip([key], cglist))  # noqa B905
 
         # print(param_dict)
-
+        logger.info("  - returning AggData")
         return AggData(
             variable=key,
-            cat_param=CatParams(**param_dict.get(key)),
-            cat_grid=CatGrids(**grid_dict.get(key)),
+            cat_param=CatParams(**param_dict.get(key)),  # type: ignore
+            cat_grid=CatGrids(**grid_dict.get(key)),  # type: ignore
             da=data_ss,
-            feature=gdf_in,
+            feature=self.f_feature,
             id_feature=self.id_feature,
             period=self.period,
         )
 
-    def _get_user_catgrids(self, key: str, da: xr.DataArray):
+    def _get_user_catgrids(self, key: str, da: xr.DataArray) -> list[dict[str, Any]]:
         """Get CatGrids from UserData."""
-        cglist = []
         cg = CatGrids(
             X_name=self.x_coord,
             Y_name=self.y_coord,
@@ -335,10 +348,9 @@ class UserCatData(UserData):
             resY=max(np.diff(da[self.y_coord].values)),
             toptobottom=_get_top_to_bottom(da, self.y_coord),
         )
-        cglist.append(cg.dict())
-        return cglist
+        return [cg.dict()]
 
-    def _get_ds_var_attrs(self, da: xr.DataArray, attr: str) -> str:
+    def _get_ds_var_attrs(self, da: xr.DataArray, attr: str) -> Any:
         """Return var attributes.
 
         Args:
@@ -346,67 +358,29 @@ class UserCatData(UserData):
             attr (str): _description_
 
         Returns:
-            str: _description_
+            Any: _description_
         """
         try:
             return da.attrs.get(attr)
         except KeyError:
             return "None"
 
-    def _get_user_catparams(self, key: str, da: xr.DataArray):
+    def _get_user_catparams(self, key: str, da: xr.DataArray) -> list[dict[str, Any]]:
         """Get CatParams from UserData."""
-        cplist = []
         cp = CatParams(
             URL="",
             varname=key,
-            long_name=self._get_ds_var_attrs(da, "long_name"),
+            long_name=str(self._get_ds_var_attrs(da, "long_name")),
             T_name=self.t_coord,
-            units=self._get_ds_var_attrs(da, "units"),
+            units=str(self._get_ds_var_attrs(da, "units")),
         )
-        cplist.append(cp.dict())
-        return cplist
+        return [cp.dict()]
 
     def prep_wght_data(self) -> WeightData:
         """Prepare and return WeightData for weight generation."""
-        ds = self._get_xr_dataset()
-
-        gdf_in = self._get_geodataframe()
-
-        gdf_in.set_crs(self.proj_feature, inplace=True)
-
-        gdf_bounds = _get_shp_bounds_w_buffer(
-            gdf=gdf_in, ds=ds, crs=self.proj_ds, lon=self.x_coord, lat=self.y_coord
-        )
-
-        is_intersect, is_degrees, is_0_360 = _check_for_intersection_nc(
-            ds=ds,
-            x_name=self.x_coord,
-            y_name=self.y_coord,
-            proj=self.proj_ds,
-            gdf=gdf_in,
-        )
-
-        if bool((not is_intersect) & is_degrees & (not is_0_360)):  # rotate
-            print("rotating")
-            ds.coords[self.x_coord] = (ds.coords[self.x_coord] + 180) % 360 - 180
-            ds = ds.sortby(ds[self.x_coord])
-
-        # calculate toptobottom (order of latitude coords)
-        ttb = _get_top_to_bottom(ds, self.y_coord)
-
-        subset_dict = build_subset(
-            bounds=gdf_bounds,
-            xname=self.x_coord,
-            yname=self.y_coord,
-            tname=self.t_coord,
-            toptobottom=ttb,
-            date_min=self.period[0],
-        )
-
-        # print(subset_dict)
-
-        data_ss_wght = ds.sel(**subset_dict)
-
+        logger.info("Weight Data preparation - beginning")
+        data_ss_wght = self.ds.sel(**self.weight_subset_dict)  # type: ignore
+        logger.info("  - calculating grid-cell polygons")
         start = time.perf_counter()
         grid_poly = _get_cells_poly(
             xr_a=data_ss_wght,
@@ -416,27 +390,15 @@ class UserCatData(UserData):
         )
         end = time.perf_counter()
         print(
-            f"Generating grid-cell bounds finished in {round(end-start, 2)} second(s)"
+            f"Generating grid-cell polygons finished in {round(end-start, 2)} second(s)"
+        )
+        logger.info(
+            f"Generating grid-cell polygons finished in {round(end-start, 2)} second(s)"
         )
         return WeightData(
-            feature=gdf_in, id_feature=self.id_feature, grid_cells=grid_poly
+            feature=self.f_feature, id_feature=self.id_feature, grid_cells=grid_poly
         )
 
-    def _get_geodataframe(self):
-        """Get GeoDataFrame."""
-        if not isinstance(self.f_feature, gpd.GeoDataFrame):
-            return gpd.read_file(self.f_feature)
-        else:
-            return self.f_feature
-
-    def _get_xr_dataset(self):
-        """Get xarray.Dataset."""
-        if not isinstance(self.ds, xr.Dataset):
-            ds = xr.open_dataset(self.ds)
-        else:
-            ds = self.ds
-        return ds
-
 
 class UserTiffData(UserData):
     """Instance of UserData for zonal stats processing of geotiffs."""
@@ -444,7 +406,7 @@ class UserTiffData(UserData):
     def __init__(
         self,
         var: str,
-        ds: Union[str, xr.Dataset],
+        ds: Union[str, xr.DataArray, xr.Dataset],
         proj_ds: Any,
         x_coord: str,
         y_coord: str,
@@ -475,10 +437,10 @@ class UserTiffData(UserData):
         self.y_coord = y_coord
         self.bname = bname
         self.band = band
-        self.toptobottom = None
-        self._get_rxr_dataset(ds)
+        self.ds = _get_rxr_dataset(ds)
+        self.toptobottom = _get_top_to_bottom(self.ds, self.y_coord)
         self.proj_ds = proj_ds
-        self._get_geodataframe(f_feature)
+        self.f_feature = _get_geodataframe(f_feature)
         self.id_feature = id_feature
         self.proj_feature = proj_feature
         self.f_feature = self.f_feature.sort_values(self.id_feature).dissolve(
@@ -490,7 +452,7 @@ class UserTiffData(UserData):
         self._check_yname()
         self._check_band()
         self._check_crs()
-        self._get_toptobottom()
+        self.toptobottom = _get_top_to_bottom(data=self.ds, y_coord=self.y_coord)
 
     def get_vars(self) -> list[str]:
         """Return list of varnames."""
@@ -523,7 +485,7 @@ class UserTiffData(UserData):
             band=self.band,
         )
 
-        data_ss = self.ds.sel(**subset_dict)  # .rio.interpolate_na()
+        data_ss: xr.DataArray = self.ds.sel(**subset_dict)  # type: ignore
         # tstrt = time.perf_counter()
         # data_ss = data_ss.rio.interpolate_na(method='nearest')
         # tend = time.perf_counter()
@@ -535,35 +497,32 @@ class UserTiffData(UserData):
 
         return AggData(
             variable=key,
-            cat_param=CatParams(**param_dict.get(key)),
-            cat_grid=CatGrids(**grid_dict.get(key)),
+            cat_param=CatParams(**param_dict.get(key)),  # type: ignore
+            cat_grid=CatGrids(**grid_dict.get(key)),  # type: ignore
             da=data_ss,
-            feature=self.f_feature,
+            feature=self.f_feature.copy(),
             id_feature=self.id_feature,
             period=["None", "None"],
         )
 
-    def _get_toptobottom(self):
-        """Generate toptobottom."""
-        yy = self.ds.coords[self.y_coord].values
-        self.toptobottom = yy[0] <= yy[-1]
-
-    def _check_xname(self):
+    def _check_xname(self: "UserTiffData") -> None:
         """Validate xname."""
         if self.x_coord not in self.ds.coords:
             raise ValueError(f"xname:{self.x_coord} not in {self.ds.coords}")
 
-    def _check_yname(self):
+    def _check_yname(self: "UserTiffData") -> None:
         """Validate yname."""
         if self.y_coord not in self.ds.coords:
             raise ValueError(f"yname:{self.y_coord} not in {self.ds.coords}")
 
-    def _check_band(self):
+    def _check_band(self: "UserTiffData") -> None:
         """Validate band name."""
         if self.bname not in self.ds.coords:
-            raise ValueError(f"band:{self.bname} not in {self.ds.coords}")
+            raise ValueError(
+                f"band:{self.bname} not in {self.ds.coords} or {self.ds.data_vars}"
+            )
 
-    def _check_crs(self):
+    def _check_crs(self: "UserTiffData") -> None:
         """Validate crs."""
         crs = CRS.from_user_input(self.proj_ds)
         if not isinstance(crs, CRS):
@@ -572,21 +531,7 @@ class UserTiffData(UserData):
         if not isinstance(crs2, CRS):
             raise ValueError(f"ds_proj:{self.proj_feature} not in valid")
 
-    def _get_rxr_dataset(self, ds: Union[str, xr.DataArray, xr.Dataset]):
-        """Get xarray.Dataset."""
-        if not isinstance(ds, (xr.Dataset, xr.DataArray)):
-            self.ds = rxr.open_rasterio(ds)
-        else:
-            self.ds = ds
-
-    def _get_geodataframe(self, f_feature: gpd.GeoDataFrame):
-        """Get GeoDataFrame."""
-        if not isinstance(f_feature, gpd.GeoDataFrame):
-            self.f_feature = gpd.read_file(f_feature)
-        else:
-            self.f_feature = f_feature
-
-    def _get_user_catgrids(self, da: xr.DataArray):
+    def _get_user_catgrids(self: "UserTiffData", da: xr.DataArray) -> dict[str, Any]:
         """Get CatGrids from UserData."""
         cg = CatGrids(
             X_name=self.x_coord,
@@ -598,7 +543,7 @@ class UserTiffData(UserData):
         )
         return cg.dict()
 
-    def _get_user_catparams(self, da: xr.DataArray):
+    def _get_user_catparams(self: "UserTiffData", da: xr.DataArray) -> dict[str, Any]:
         """Get CatParams from UserData."""
         cp = CatParams(
             URL="",
@@ -609,7 +554,7 @@ class UserTiffData(UserData):
         )
         return cp.dict()
 
-    def _get_ds_var_attrs(self, da: xr.DataArray, attr: str) -> str:
+    def _get_ds_var_attrs(self: "UserTiffData", da: xr.DataArray, attr: str) -> str:
         """Return var attributes.
 
         Args:
@@ -620,6 +565,6 @@ class UserTiffData(UserData):
             str: _description_
         """
         try:
-            return da.attrs.get(attr)
+            return str(da.attrs.get(attr))
         except KeyError:
             return "na"
