@@ -17,10 +17,13 @@ from typing import Tuple
 from typing import Union
 
 import geopandas as gpd
+import geopandas.sindex
 import netCDF4
 import numpy as np
 import numpy.typing as npt
 import pandas as pd
+import pygeos
+import rioxarray as rxr
 import shapely
 import xarray as xr
 from pyproj import CRS
@@ -37,7 +40,7 @@ from gdptools.data.odap_cat_data import CatParams
 logger = logging.getLogger(__name__)
 
 
-def _get_grid_cell_sindex(grid_cells):
+def _get_grid_cell_sindex(grid_cells: gpd.GeoDataFrame) -> geopandas.sindex:
     start = time.perf_counter()
     spatial_index = grid_cells.sindex
     # print(type(spatial_index))
@@ -46,7 +49,12 @@ def _get_grid_cell_sindex(grid_cells):
     return spatial_index
 
 
-def _reproject_for_weight_calc(poly, grid_cells, grid_out_crs, wght_gen_crs):
+def _reproject_for_weight_calc(
+    poly: gpd.GeoDataFrame,
+    grid_cells: gpd.GeoDataFrame,
+    grid_out_crs: Any,
+    wght_gen_crs: Any,
+) -> Tuple[gpd.GeoDataFrame, gpd.GeoDataFrame]:
     start = time.perf_counter()
     # grid_cells.set_crs(grid_in_crs, inplace=True)
     grid_cells.to_crs(grid_out_crs, inplace=True)
@@ -59,19 +67,19 @@ def _reproject_for_weight_calc(poly, grid_cells, grid_out_crs, wght_gen_crs):
     return poly, grid_cells
 
 
-def _check_grid_cell_crs(grid_cells):
+def _check_grid_cell_crs(grid_cells: gpd.GeoDataFrame) -> None:
     if not grid_cells.crs:
         error_string = f"grid_cells don't contain a valid crs: {grid_cells.crs}"
         raise ValueError(error_string)
 
 
-def _check_feature_crs(poly):
+def _check_feature_crs(poly: gpd.GeoDataFrame) -> None:
     if not poly.crs:
         error_string = f"polygons don't contain a valid crs: {poly.crs}"
         raise ValueError(error_string)
 
 
-def _check_poly_idx(poly, poly_idx):
+def _check_poly_idx(poly: gpd.GeoDataFrame, poly_idx: str) -> None:
     if poly_idx not in poly.columns:
         error_string = (
             f"Error: poly_idx ({poly_idx}) is not found in the poly ({poly.columns})"
@@ -137,120 +145,112 @@ def _get_cells_poly(  # noqa
 
     Returns:
         gpd.GeoDataFrame: _description_
+
+        grid-cell polygons are calculated as follows:
+
+        1) The polygons surrounding each node, where for each node at
+           (i, j) the 4 surrounding polygons tpoly1a, tpoly2a, tpoly3a
+           tpoly4a are calculated.
+
+        (i-1, j+1)    (i, j+1)  (i+1, j+1)
+            *...........*...........*
+            .           .           .
+            .           .           .
+            . (tpoly3a) . (tpoly2a) .
+            .           .           .
+            .           .           .
+        (i-1, j)      (i, j)    (i+1, j)
+            *...........*...........*
+            .           .           .
+            .           .           .
+            . (tpoly4a) . (tpoly1a) .
+            .           .           .
+            .           .           .
+            *...........*...........*
+        (i-1, j-1)    (i, j-1)  (i+1, j-1)
+
+        2) The centroid is calculated for each of the 4 polygons
+           in step 1, from with the bonding polygon of the node
+           is determined.
+            *..........*..........*
+            .          .          .
+            .          .          .
+            .    p3----------p2   .
+            .    |     .      |   .
+            .    |     .      |   .
+            *....|.....*......|...*
+            .    |     .      |   .
+            .    |     .      |   .
+            .    p4----------p1   .
+            .          .          .
+            .          .          .
+            *..........*..........*
+
+        The grid-cell polygon surounding the node/vertex at (i, j) is
+        [p1, p2, p3, p4, p1]
+
+        This is to account for both rectangular and non-rectangular
+        grid geometries.
     """
     tlon = xr_a[x]
     tlat = xr_a[y]
-    in_crs = _get_crs(crs_in)
-    # if type(crs_in) == int:
-    #     in_crs = CRS.from_epsg(crs_in)
-    # elif type(crs_in) == str:
-    #     in_crs = CRS.from_proj4(crs_in)
+    in_crs = crs_in
 
-    # out_crs = _get_crs(4326)
     lon, lat = np.meshgrid(tlon, tlat)
     poly = []
     if verbose:
         logger.info("calculating surrounding cell vertices")
     start = time.perf_counter()
-    lon_n = [
-        lon[i, j]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_jm1 = [
-        lon[i, j - 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_ip1_jm1 = [
-        lon[i + 1, j - 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_ip1 = [
-        lon[i + 1, j]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_ip1_jp1 = [
-        lon[i + 1, j + 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_jp1 = [
-        lon[i, j + 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_im1_jp1 = [
-        lon[i - 1, j + 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_im1 = [
-        lon[i - 1, j]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
-    lon_im1_jm1 = [
-        lon[i - 1, j - 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lon.shape[1] - 1)
-    ]
 
-    lat_n = [
-        lat[i, j]
+    tpoly1a = [
+        [
+            [lon[i, j], lat[i, j]],
+            [lon[i, j - 1], lat[i, j - 1]],
+            [lon[i + 1, j - 1], lat[i + 1, j - 1]],
+            [lon[i + 1, j], lat[i + 1, j]],
+        ]
         for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
+        for j in range(1, lon.shape[1] - 1)
     ]
-    lat_jm1 = [
-        lat[i, j - 1]
+    tpoly2a = [
+        [
+            [lon[i, j], lat[i, j]],
+            [lon[i + 1, j], lat[i + 1, j]],
+            [lon[i + 1, j + 1], lat[i + 1, j + 1]],
+            [lon[i, j + 1], lat[i, j + 1]],
+        ]
         for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
+        for j in range(1, lon.shape[1] - 1)
     ]
-    lat_ip1_jm1 = [
-        lat[i + 1, j - 1]
+    tpoly3a = [
+        [
+            [lon[i, j], lat[i, j]],
+            [lon[i, j + 1], lat[i, j + 1]],
+            [lon[i - 1, j + 1], lat[i - 1, j + 1]],
+            [lon[i - 1, j], lat[i - 1, j]],
+        ]
         for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
+        for j in range(1, lon.shape[1] - 1)
     ]
-    lat_ip1 = [
-        lat[i + 1, j]
+    tpoly4a = [
+        [
+            [lon[i, j], lat[i, j]],
+            [lon[i - 1, j], lat[i - 1, j]],
+            [lon[i - 1, j - 1], lat[i - 1, j - 1]],
+            [lon[i, j - 1], lat[i, j - 1]],
+        ]
         for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
-    ]
-    lat_ip1_jp1 = [
-        lat[i + 1, j + 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
-    ]
-    lat_jp1 = [
-        lat[i, j + 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
-    ]
-    lat_im1_jp1 = [
-        lat[i - 1, j + 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
-    ]
-    lat_im1 = [
-        lat[i - 1, j]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
-    ]
-    lat_im1_jm1 = [
-        lat[i - 1, j - 1]
-        for i in range(1, lon.shape[0] - 1)
-        for j in range(1, lat.shape[1] - 1)
+        for j in range(1, lon.shape[1] - 1)
     ]
     end = time.perf_counter()
     if verbose:
         logger.info(
-            f"finished calculating surrounding cell vertices in {round(end-start, 2)} second(s)"
+            "finished calculating surrounding cell vertices in"
+            f" {round(end-start, 2)} second(s)"
         )
 
     # print(len(lon_n), len(lat_n), type(lon_n), np.shape(lon_n))
-    numcells = len(lon_n)
+    numcells = len(tpoly1a)
     index = np.array(range(numcells))
     i_index = np.empty(numcells)
     j_index = np.empty(numcells)
@@ -262,152 +262,41 @@ def _get_cells_poly(  # noqa
             count += 1
 
     if verbose:
-        logger.info("calculating cell 1 centroids")
+        logger.info("calculating centroids")
+
     start = time.perf_counter()
-    tpoly_1_lon = [
-        [lon_n[i], lon_jm1[i], lon_ip1_jm1[i], lon_ip1[i]] for i in range(numcells)
-    ]
-    tpoly_1_lat = [
-        [lat_n[i], lat_jm1[i], lat_ip1_jm1[i], lat_ip1[i]] for i in range(numcells)
-    ]
-    # print(len(tpoly_1_lon), tpoly_1_lon[0])
-    newp = [
-        Polygon(zip(tpoly_1_lon[i], tpoly_1_lat[i]))  # noqa B905
-        for i in range(numcells)
-    ]
-    p1 = [p.centroid for p in newp]
-    # print(type(newp), newp[0], len(p1))
+    tpoly1 = [pygeos.polygons(tpoly1a)]
+    p1 = pygeos.get_coordinates(pygeos.centroid(tpoly1))
+
+    tpoly2 = [pygeos.polygons(tpoly2a)]
+    p2 = pygeos.get_coordinates(pygeos.centroid(tpoly2))
+
+    tpoly3 = [pygeos.polygons(tpoly3a)]
+    p3 = pygeos.get_coordinates(pygeos.centroid(tpoly3))
+
+    tpoly4 = [pygeos.polygons(tpoly4a)]
+    p4 = pygeos.get_coordinates(pygeos.centroid(tpoly4))
     end = time.perf_counter()
+
     if verbose:
         logger.info(
-            f"finished calculating cell 1 centroids in {round(end-start, 2)} second(s)"
+            "finished calculating surrounding cell vertices "
+            f" in {round(end-start, 2)} second(s)"
         )
-    del tpoly_1_lat, tpoly_1_lon
-
-    if verbose:
-        logger.info("calculating cell 2 centroids")
-    start = time.perf_counter()
-    tpoly_2_lon = [
-        [lon_n[i], lon_ip1[i], lon_ip1_jp1[i], lon_jp1[i]] for i in range(numcells)
-    ]
-    tpoly_2_lat = [
-        [lat_n[i], lat_ip1[i], lat_ip1_jp1[i], lat_jp1[i]] for i in range(numcells)
-    ]
-    # print(len(tpoly_2_lon), tpoly_2_lon[0])
-    newp = [
-        Polygon(zip(tpoly_2_lon[i], tpoly_2_lat[i]))  # noqa B905
-        for i in range(numcells)
-    ]
-    p2 = [p.centroid for p in newp]
-    end = time.perf_counter()
-    if verbose:
-        logger.info(
-            f"finished calculating cell 2 centroids in {round(end-start, 2)} second(s)"
-        )
-
-    del tpoly_2_lat, tpoly_2_lon, newp
-
-    if verbose:
-        logger.info("calculating cell 3 centroids")
-    start = time.perf_counter()
-    tpoly_3_lon = [
-        [lon_n[i], lon_jp1[i], lon_im1_jp1[i], lon_im1[i]] for i in range(numcells)
-    ]
-    tpoly_3_lat = [
-        [lat_n[i], lat_jp1[i], lat_im1_jp1[i], lat_im1[i]] for i in range(numcells)
-    ]
-    # print(len(tpoly_3_lon), tpoly_3_lon[0])
-    newp = [
-        Polygon(zip(tpoly_3_lon[i], tpoly_3_lat[i]))  # noqa B905
-        for i in range(numcells)
-    ]
-    p3 = [p.centroid for p in newp]
-    end = time.perf_counter()
-    if verbose:
-        logger.info(
-            f"finished calculating cell 3 centroids in {round(end-start, 2)} second(s)"
-        )
-
-    del tpoly_3_lat, tpoly_3_lon, newp
-
-    if verbose:
-        logger.info("calculating cell 4 centroids")
-    start = time.perf_counter()
-    tpoly_4_lon = [
-        [lon_n[i], lon_im1[i], lon_im1_jm1[i], lon_jm1[i]] for i in range(numcells)
-    ]
-    tpoly_4_lat = [
-        [lat_n[i], lat_im1[i], lat_im1_jm1[i], lat_jm1[i]] for i in range(numcells)
-    ]
-    # print(len(tpoly_3_lon), tpoly_3_lon[0])
-    newp = [
-        Polygon(zip(tpoly_4_lon[i], tpoly_4_lat[i]))  # noqa B905
-        for i in range(numcells)
-    ]
-    p4 = [p.centroid for p in newp]
-    end = time.perf_counter()
-    if verbose:
-        logger.info(
-            f"finished calculating cell 4 centroids in {round(end-start, 2)} second(s)"
-        )
-
-    del tpoly_4_lat, tpoly_4_lon, newp
-    del (
-        lon_n,
-        lon_jm1,
-        lon_ip1_jm1,
-        lon_ip1,
-        lon_ip1_jp1,
-        lon_jp1,
-        lon_im1_jp1,
-        lon_im1,
-        lon_im1_jm1,
-    )
-    del (
-        lat_n,
-        lat_jm1,
-        lat_ip1_jm1,
-        lat_ip1,
-        lat_ip1_jp1,
-        lat_jp1,
-        lat_im1_jp1,
-        lat_im1,
-        lat_im1_jm1,
-    )
-
-    if verbose:
-        logger.info("creating bounding polygons")
-    start = time.perf_counter()
-    lon_point_list = [[p1[i].x, p2[i].x, p3[i].x, p4[i].x] for i in range(numcells)]
-    lat_point_list = [[p1[i].y, p2[i].y, p3[i].y, p4[i].y] for i in range(numcells)]
+    lon_point_list = [[p1[i][0], p2[i][0], p3[i][0], p4[i][0]] for i in range(numcells)]
+    lat_point_list = [[p1[i][1], p2[i][1], p3[i][1], p4[i][1]] for i in range(numcells)]
     poly = [
         Polygon(zip(lon_point_list[i], lat_point_list[i]))  # noqa B905
         for i in range(numcells)
     ]
-    end = time.perf_counter()
-    if verbose:
-        logger.info(
-            f"finished creating bounding polygons in {round(end-start, 2)} second(s)"
-        )
-
-    if verbose:
-        logger.info("reprojecting cells")
-    start = time.perf_counter()
-    # grd_shp = xr_a[var].values.shape
     df = pd.DataFrame({"i_index": i_index, "j_index": j_index})
-    gmcells = gpd.GeoDataFrame(df, index=index, geometry=poly, crs=in_crs)
-    # gmcells.to_crs(crs=out_crs, inplace=True)
-    end = time.perf_counter()
-    if verbose:
-        logger.info(f"finished reprojecting cells in {round(end-start, 2)} second(s)")
-
-    return gmcells
+    return gpd.GeoDataFrame(df, index=index, geometry=poly, crs=in_crs)
 
 
 def _build_subset_cat(
     cat_params: CatParams,
     cat_grid: CatGrids,
-    bounds: npt.NDArray[np.double],
+    bounds: Tuple[np.double, np.double, np.double, np.double],
     date_min: str,
     date_max: Optional[str] = None,
 ) -> Dict[Any, Any]:
@@ -479,7 +368,7 @@ def _read_shp_file(shp_file: Union[str, Path, gpd.GeoDataFrame]) -> gpd.GeoDataF
 
 def _get_shp_file(
     shp_file: gpd.GeoDataFrame, cat_grid: CatGrids, is_degrees: bool
-) -> Tuple[gpd.GeoDataFrame, npt.NDArray[np.double]]:
+) -> Tuple[gpd.GeoDataFrame, Tuple[np.double, np.double, np.double, np.double]]:
     """Return GeoDataFrame and bounds of shapefile.
 
     Args:
@@ -494,9 +383,9 @@ def _get_shp_file(
     gdf = shp_file.to_crs(cat_grid.proj)
     # buffer polygons bounding box by twice max resolution of grid
     bbox = box(*gdf.total_bounds)
-    gdf_bounds = bbox.buffer(2 * max(cat_grid.resX, cat_grid.resY)).bounds
+    gdf_bounds = bbox.buffer(2.0 * max(cat_grid.resX, cat_grid.resY)).bounds  # type: ignore
     if is_degrees and (gdf_bounds[0] < -180.0) & (gdf_bounds[2] > 180.0):
-        newxmax = 180 - (abs(gdf_bounds[0]) - 180.0)
+        newxmax = 180.0 - (abs(gdf_bounds[0]) - 180.0)
         newxmin = -180.0 + (abs(gdf_bounds[2]) - 180.0)
         gdf_bounds = (newxmin, gdf_bounds[1], newxmax, gdf_bounds[3])
         print("Warning: The feature data crosses the anitmeridian.")
@@ -519,7 +408,7 @@ def _is_degrees(ds: xr.Dataset, cat_grid: CatGrids) -> bool:
     xvals = ds[cat_grid.X_name]
     t_attr = xvals.attrs
     units = t_attr.get("units")
-    return bool(re.search("degrees", units, flags=re.IGNORECASE))
+    return bool(re.search("degrees", str(units), flags=re.IGNORECASE))
 
 
 def _is_degrees_nc(ds: xr.Dataset, x_name: str) -> bool:
@@ -544,7 +433,7 @@ def _is_degrees_nc(ds: xr.Dataset, x_name: str) -> bool:
             "a unit attribute per cf-conventions"
         )
 
-    return bool(re.search("degrees", units, flags=re.IGNORECASE))
+    return bool(re.search("degrees", str(units), flags=re.IGNORECASE))
 
 
 def _is_lon_0_360(vals: npt.NDArray[np.double]) -> bool:
@@ -568,7 +457,11 @@ def _is_lon_0_360(vals: npt.NDArray[np.double]) -> bool:
 
 
 def _get_shp_bounds_w_buffer(
-    gdf: gpd.GeoDataFrame, ds: xr.Dataset, crs: Any, lon: str, lat: str
+    gdf: gpd.GeoDataFrame,
+    ds: Union[xr.DataArray, xr.Dataset],
+    crs: Any,
+    lon: str,
+    lat: str,
 ) -> npt.NDArray[np.double]:
     """Return bounding box based on 2 * max(ds.dx, ds.dy).
 
@@ -585,7 +478,7 @@ def _get_shp_bounds_w_buffer(
     bbox = box(*gdf.to_crs(crs).total_bounds)
     return np.asarray(
         bbox.buffer(
-            2 * max(max(np.diff(ds[lat].values)), max(np.diff(ds[lon].values)))
+            2 * max(max(np.diff(ds[lat].values)), max(np.diff(ds[lon].values)))  # type: ignore
         ).bounds
     )
 
@@ -609,9 +502,7 @@ def _check_for_intersection(
     is_intersect = True
     is_0_360 = False
     ds_url = cat_params.URL
-    ds = xr.open_dataset(
-        ds_url + "#fillmismatch", decode_coords=True, chunks={}
-    )  # type:ignore
+    ds = xr.open_dataset(ds_url + "#fillmismatch", decode_coords=True, chunks={})
     xvals = ds[cat_grid.X_name]
     yvals = ds[cat_grid.Y_name]
     minx = xvals.values.min()
@@ -694,7 +585,7 @@ def _check_for_intersection_nc(
 def _get_data_via_catalog(
     cat_params: CatParams,
     cat_grid: CatGrids,
-    bounds: npt.NDArray[np.double],
+    bounds: Tuple[np.double, np.double, np.double, np.double],
     begin_date: str,
     end_date: Optional[str] = None,
     rotate_lon: Optional[bool] = False,
@@ -713,9 +604,7 @@ def _get_data_via_catalog(
         xr.DataArray: _description_
     """
     ds_url = cat_params.URL
-    ds = xr.open_dataset(
-        ds_url + "#fillmismatch", decode_coords=True, chunks={}
-    )  # type:ignore
+    ds = xr.open_dataset(ds_url + "#fillmismatch", decode_coords=True, chunks={})
     if rotate_lon:
         lon = cat_grid.X_name
         ds.coords[lon] = (ds.coords["lon"] + 180) % 360 - 180
@@ -730,7 +619,7 @@ def _get_data_via_catalog(
     # -dealing-with-fillvalue-type-mismatch-error/61
 
     varname = cat_params.varname
-    return ds[varname].sel(**ss_dict)  # type: ignore
+    return ds[varname].sel(**ss_dict)
 
 
 def _get_wieght_df(wght_file: Union[str, pd.DataFrame], poly_idx: str) -> pd.DataFrame:
@@ -765,8 +654,8 @@ def _date_range(p_start: str, p_end: str, intv: int) -> Iterator[str]:
     yield end.strftime("%Y-%m-%d")
 
 
-def _get_catalog_time_increment(param: dict) -> Tuple[int, str]:
-    interval = param.get("interval").split(" ")
+def _get_catalog_time_increment(param: dict) -> Tuple[int, str]:  # type: ignore
+    interval = str(param.get("interval")).split(" ")
     return int(interval[0]), str(interval[1])
 
 
@@ -800,8 +689,8 @@ def _quadrat_cut_geometry(
     west, south, east, north = geometry.bounds
     x_num = int(np.ceil((east - west) / quadrat_width) + 1)
     y_num = int(np.ceil((north - south) / quadrat_width) + 1)
-    x_points = np.linspace(west, east, num=max(x_num, min_num))
-    y_points = np.linspace(south, north, num=max(y_num, min_num))
+    x_points = np.linspace(west, east, num=np.max(x_num, min_num))
+    y_points = np.linspace(south, north, num=np.max(y_num, min_num))
 
     # create a quadrat grid of lines at each of the evenly spaced points
     vertical_lines = [
@@ -827,7 +716,7 @@ def _quadrat_cut_geometry(
     return MultiPolygon(geometries)
 
 
-def _get_default_val(native_dtype: np.dtype):
+def _get_default_val(native_dtype: np.dtype):  # type: ignore
     """Get default fill value based on gridded data dtype.
 
     Args:
@@ -852,9 +741,16 @@ def _get_default_val(native_dtype: np.dtype):
     return dfval
 
 
-def _get_interp_array(n_geo: int, nts: int, native_dtype: np.dtype, default_val: Any):
+def _get_interp_array(
+    n_geo: int,
+    nts: int,
+    native_dtype: Union[np.dtype[np.double], np.dtype[np.int_]],
+    default_val: Any,
+) -> Union[npt.NDArray[np.int_], npt.NDArray[np.double]]:
     """Get array for interpolation based on the dtype of the gridded data.
 
+    _extended_summary_
+
     Args:
         n_geo (int): _description_
         nts (int): _description_
@@ -865,7 +761,7 @@ def _get_interp_array(n_geo: int, nts: int, native_dtype: np.dtype, default_val:
         TypeError: _description_
 
     Returns:
-        _type_: _description_
+        Union[npt.NDArray[np.int_], npt.NDArray[np.double]]: _description_
     """
     if native_dtype.kind == "i":
         # val_interp = np.empty((nts, n_geo), dtype=np.dtype("int64"))
@@ -897,5 +793,30 @@ def _get_top_to_bottom(data: Union[xr.Dataset, xr.DataArray], y_coord: str) -> b
         bool: _description_
     """
     yy = data.coords[y_coord].values
-    ttb: bool = yy[0] <= yy[-1]
-    return ttb
+    return bool(yy[0] <= yy[-1])
+
+
+def _get_xr_dataset(ds: Union[str, xr.Dataset]) -> xr.Dataset:
+    """Get xarray.Dataset."""
+    if not isinstance(ds, xr.Dataset):
+        return xr.open_dataset(ds)
+    else:
+        return ds
+
+
+def _get_rxr_dataset(
+    ds: Union[str, xr.DataArray, xr.Dataset]
+) -> Union[xr.DataArray, xr.Dataset]:
+    """Get xarray.Dataset."""
+    if not isinstance(ds, (xr.Dataset, xr.DataArray)):
+        return rxr.open_rasterio(ds)  # type: ignore
+    else:
+        return ds
+
+
+def _get_geodataframe(f_feature: gpd.GeoDataFrame) -> gpd.GeoDataFrame:
+    """Get GeoDataFrame."""
+    if not isinstance(f_feature, gpd.GeoDataFrame):
+        return gpd.read_file(f_feature)
+    else:
+        return f_feature
