@@ -3,15 +3,16 @@ import time
 from datetime import datetime
 from pathlib import Path
 from typing import Literal
-from typing import Optional
+from typing import Union
 
 import pandas as pd
 
+from gdptools.agg.zonal_engines import ZonalEngineDask
 from gdptools.agg.zonal_engines import ZonalEngineParallel
 from gdptools.agg.zonal_engines import ZonalEngineSerial
 from gdptools.data.user_data import UserData
 
-ZONAL_ENGINES = Literal["serial", "parallel"]
+ZONAL_ENGINES = Literal["serial", "parallel", "dask"]
 ZONAL_WRITERS = Literal["csv"]
 
 
@@ -25,7 +26,8 @@ class ZonalGen:
         zonal_writer: ZONAL_WRITERS,
         out_path: str,
         file_prefix: str,
-        append_date: Optional[bool] = False,
+        append_date: bool = False,
+        jobs: int = 1,
     ) -> None:
         """__init__ Initialize ZonalGen class.
 
@@ -37,7 +39,8 @@ class ZonalGen:
             zonal_writer (ZONAL_WRITERS): _description_
             out_path (str): _description_
             file_prefix (str): _description_
-            append_date (Optional[bool], optional): _description_. Defaults to False.
+            append_date (bool): _description_. Defaults to False.
+            jobs (int): _description_. Defaults to 1.
 
         Raises:
             FileNotFoundError: _description_
@@ -46,6 +49,7 @@ class ZonalGen:
         self._user_data = user_data
         self._zonal_engine = zonal_engine
         self._zonal_writer = zonal_writer
+        self._jobs = jobs
         self._out_path = Path(out_path)
         if not self._out_path.exists():
             raise FileNotFoundError(f"Path: {self._out_path} does not exist")
@@ -56,28 +60,30 @@ class ZonalGen:
             self._fname = f"{self._fdate}_{self._file_prefix}"
         else:
             self._fname = f"{self._file_prefix}"
-
+        self.agg: Union[ZonalEngineSerial, ZonalEngineParallel, ZonalEngineDask]
         if self._zonal_engine == "serial":
-            self.agg = ZonalEngineSerial
+            self.agg = ZonalEngineSerial()
         elif self._zonal_engine == "parallel":
-            self.agg = ZonalEngineParallel
+            self.agg = ZonalEngineParallel()
+        elif self._zonal_engine == "dask":
+            self.agg = ZonalEngineDask()
         else:
             raise TypeError(f"agg_engine: {self._zonal_engine} not in {ZONAL_ENGINES}")
 
-    def calculate_zonal(self, categorical: Optional[bool] = False) -> pd.DataFrame:
+    def calculate_zonal(self, categorical: bool = False) -> pd.DataFrame:
         """calculate_zonal Calculates zonal statistics.
 
         _extended_summary_
 
         Args:
-            categorical (Optional[bool], optional): _description_. Defaults to False.
+            categorical (bool): _description_. Defaults to False.
 
         Returns:
             pd.DataFrame: _description_
         """
         tstrt = time.perf_counter()
-        stats = self.agg().calc_zonal_from_aggdata(
-            user_data=self._user_data, categorical=categorical
+        stats = self.agg.calc_zonal_from_aggdata(
+            user_data=self._user_data, categorical=categorical, jobs=self._jobs
         )
         if self._zonal_writer == "csv":
             fullpath = self._out_path / f"{self._fname}.csv"
