@@ -1,8 +1,11 @@
 """Tests for .helper functions."""
+from pathlib import Path
+
 import geopandas as gpd
 import pandas as pd
 import pytest
 import xarray as xr
+from pytest import FixtureRequest
 
 from gdptools.data.odap_cat_data import CatGrids
 from gdptools.data.odap_cat_data import CatParams
@@ -25,13 +28,13 @@ def get_xarray() -> xr.Dataset:
 
 
 @pytest.fixture()
-def get_file_path(tmp_path):
+def get_file_path(tmp_path: Path) -> Path:
     """Get temp file path."""
     return tmp_path / "test.csv"
 
 
 @pytest.fixture()
-def get_out_path(tmp_path):
+def get_out_path(tmp_path: Path) -> Path:
     """Get temp file output path."""
     return tmp_path
 
@@ -60,7 +63,7 @@ def catparam() -> CatParams:
 
 
 @pytest.fixture(scope="module")
-def catgrid(catparam) -> CatGrids:
+def catgrid(catparam) -> CatGrids:  # type: ignore
     """Return grid json."""
     cat_grid = "https://mikejohnson51.github.io/opendap.catalog/cat_grids.json"
     grids = pd.read_json(cat_grid)
@@ -134,7 +137,9 @@ def get_end_date() -> str:
         ),
     ],
 )
-def test_get_data_subset_catalog(cp, cg, gdf, sd, ed, name, request):
+def test_get_data_subset_catalog(
+    cp: str, cg: str, gdf: str, sd: str, ed: str, name: str, request: FixtureRequest
+) -> None:
     """Test subset catalog."""
     ds = get_data_subset_catalog(
         cat_params=request.getfixturevalue(cp),
